@@ -26,6 +26,9 @@
 (defconstant ARRAY-TOTAL-SIZE-LIMIT 4294967296)
 
 
+(defconstant CHAR-CODE-LIMIT 4294967296)
+
+
 (defun identity (x) x)
 
 
@@ -66,6 +69,9 @@
 (defun isqrt (num)
   (values (floor (sqrt num))))
 
+(defun conjugate (num)
+  (complex (realpart num) (- (imagpart num))))
+
 
 (defun first (list) (nth 0 list))
 (defun second (list) (nth 1 list))
@@ -294,6 +300,10 @@
     nil))
 
 
+(defun char-int (ch)
+  (char-code ch))
+
+
 (defun string-upcase (s)
   (let* ((str (string s))
 	 (ret (make-string (length str))))
@@ -360,6 +370,9 @@
 (defun symbolp (obj)
   (typep obj 'symbol))
 
+(defun keywordp (obj)
+  (typep obj 'keyword))
+
 (defun functionp (obj)
   (typep obj 'function))
 
@@ -375,6 +388,9 @@
 (defun floatp (obj)
   (typep obj 'float))
 
+(defun complexp (obj)
+  (typep obj 'complex))
+
 (defun characterp (obj)
   (typep obj 'character))
 
@@ -442,14 +458,14 @@
 
 
 
-(defun terpri ()
-  (write-char #\newline)
+(defun terpri (&optional (out *standard-output*))
+  (write-char #\newline out)
   nil)
 
 
-(defun write-line (string)
-  (write-string string)
-  (terpri)
+(defun write-line (string &optional (out *standard-output*))
+  (write-string string out)
+  (terpri out)
   string)
 
 
